"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseBearerToken = exports.signDeviceToken = exports.signUserToken = exports.sign = void 0;
const config_1 = require("../config");
const roleRepository_1 = require("../database/repositories/roleRepository");
const utils_1 = require("./utils");
const service_common_1 = require("@crosslab/service-common");
const jose_1 = require("jose");
/**
 * This function signs a JWT.
 * @param payload Payload of the JWT.
 * @param key Key used for signing.
 * @param expirationTime Expiration time of the JWT.
 * @returns The signed JWT.
 */
async function sign(payload, key, expirationTime) {
    return await new jose_1.SignJWT(payload)
        .setProtectedHeader({ alg: key.alg, kid: key.uuid })
        .setIssuer(config_1.config.SECURITY_ISSUER)
        .setIssuedAt()
        .setAudience(config_1.config.SECURITY_AUDIENCE)
        .setExpirationTime(expirationTime)
        .sign(await (0, jose_1.importJWK)(key.private_key, key.alg));
}
exports.sign = sign;
/**
 * This function signs a token for the user for the use of the microservice architecture.
 * @param user User for which the token should be signed.
 * @param activeKey Active key used for signing the token.
 * @returns The signed token.
 */
async function signUserToken(user, activeKey, scopes) {
    return await sign({
        url: (0, utils_1.userUrlFromId)(user.uuid),
        username: user.username,
        scopes: scopes ??
            user.roles
                .map((role) => role.scopes.map((scope) => scope.name))
                .flat(1)
                .filter((value, index, array) => array.indexOf(value) === index),
    }, activeKey.key, '2h');
}
exports.signUserToken = signUserToken;
/**
 * This function signs a token for the device for the use of the microservice architecture.
 * @param deviceUrl The url of the device for which the token should be signed.
 * @param user User for which the token should be signed.
 * @param activeKey Active key used for signing the token.
 * @returns The signed token.
 */
async function signDeviceToken(deviceUrl, user, activeKey) {
    const roleModelDevice = await roleRepository_1.roleRepository.findOneOrFail({
        where: {
            name: 'device',
        },
    });
    return await sign({
        url: (0, utils_1.userUrlFromId)(user.uuid),
        username: user.username,
        device: deviceUrl,
        scopes: roleModelDevice.scopes.map((scope) => scope.name),
    }, activeKey.key, '2h');
}
exports.signDeviceToken = signDeviceToken;
/**
 * This function parses bearer token from the Authorization parameter
 * @param authorization Authorization parameter from request.
 * @returns String representation of the token.
 */
function parseBearerToken(authorization) {
    const regex = /^Bearer (\S*)$/;
    if (!authorization) {
        throw new service_common_1.MissingParameterError(`Authorization parameter is missing`, 401);
    }
    const match = authorization.match(regex);
    if (!match || match.length !== 2) {
        throw new service_common_1.MalformedParameterError(`Authorization parameter is malformed`, 401);
    }
    return match[1];
}
exports.parseBearerToken = parseBearerToken;
//# sourceMappingURL=auth.js.map