"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loginLocal = exports.loginTui = void 0;
const roleRepository_1 = require("../database/repositories/roleRepository");
const tokenRepository_1 = require("../database/repositories/tokenRepository");
const userRepository_1 = require("../database/repositories/userRepository");
const errors_1 = require("../types/errors");
const bcryptjs_1 = require("bcryptjs");
const ldapts_1 = require("ldapts");
/**
 * This function creates a token for a user.
 * @param userModel The user the token should be created for.
 * @param expiresIn Time until the token expires (default: one hour).
 * @returns Newly created token.
 */
async function createUserToken(userModel, expiresIn = 3600000) {
    const tokenModel = await tokenRepository_1.tokenRepository.create({
        user: userModel.username,
        scopes: [],
        roles: userModel.roles.map((role) => role.name),
        expiresOn: new Date(Date.now() + expiresIn).toISOString(),
    });
    userModel.tokens.push(tokenModel);
    await userRepository_1.userRepository.save(userModel);
    return tokenModel;
}
/**
 * This function creates a new user for logging in via the TUI ldap system.
 * @param username The username of the client for the TUI ldap system.
 * @returns The newly created TUI user.
 */
async function createUserTUI(username) {
    const userModel = await userRepository_1.userRepository.create({
        username: 'tui:' + username,
        password: '',
    });
    const roleModelUser = await roleRepository_1.roleRepository.findOneOrFail({
        where: {
            name: 'user',
        },
    });
    userModel.roles = [roleModelUser];
    userModel.tokens = [];
    await userRepository_1.userRepository.save(userModel);
    return userModel;
}
/**
 * This function attempts to login at the TUI Ldap system with the provided credentials.
 * @param username Username of the client.
 * @param password Password of the client.
 * @throws {LdapAuthenticationError} Thrown if using invalid credentials.
 * @throws {LdapBindError} Thrown if ldap bind fails.
 * @throws {LdapError} Thrown if ldap search does not return any entries.
 * @returns A token on successful login.
 */
async function loginTui(username, password) {
    // Initialize Ldap Client
    const client = new ldapts_1.Client({
        url: 'ldaps://ldapauth.tu-ilmenau.de:636',
        tlsOptions: {
            minVersion: 'TLSv1',
        },
        timeout: 10000,
    });
    // Bind Ldap Client
    const dn = `cn=${username},ou=user,o=uni`;
    try {
        await client.bind(dn, password);
    }
    catch (err) {
        if (err.code === 49) {
            throw new errors_1.LdapAuthenticationError('TUI ldap authentication failed because of invalid login credentials', 401);
        }
        else {
            // TODO add better error messages for the other errors (e.g. host unreachable)
            throw new errors_1.LdapBindError('TUI ldap bind operation failed', 500);
        }
    }
    // Make Ldap Search Request
    const searchResult = await client.search(dn);
    if (searchResult.searchEntries.length === 0) {
        throw new errors_1.LdapError(`No entries found for dn "cn=${username},ou=user,o=uni" during ldap search`, 401);
    }
    // Find User with matching Username
    let userModel = await userRepository_1.userRepository.findOne({
        where: {
            username: 'tui:' + username,
        },
    });
    // Create new User if no User was found
    if (!userModel) {
        console.log(`User tui:${username} not found, creating new user`);
        userModel = await createUserTUI(username);
    }
    return await createUserToken(userModel);
}
exports.loginTui = loginTui;
/**
 * This function attempts to login with the provided credentials.
 * @param username Username of the client.
 * @param password Password of the client.
 * @throws {AuthenticationError} Thrown if credentials are invalid.
 * @throws {InconsistentDatabaseError} Thrown if user found in database has no password.
 * @returns A token on successful login.
 */
async function loginLocal(username, password) {
    const userModel = await userRepository_1.userRepository.findOne({
        where: {
            username: `local:${username}`,
        },
    });
    if (!userModel)
        throw new errors_1.AuthenticationError(`Invalid login credentials`, 401);
    if (!userModel.password)
        throw new errors_1.AuthenticationError(`Password authentication not possible for this user`, 401);
    if (!(await (0, bcryptjs_1.compare)(password, userModel.password)))
        throw new errors_1.AuthenticationError(`Invalid login credentials`, 401);
    return await createUserToken(userModel);
}
exports.loginLocal = loginLocal;
//# sourceMappingURL=login.js.map