"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAuth = void 0;
const activeKeyRepository_1 = require("../database/repositories/activeKeyRepository");
const tokenRepository_1 = require("../database/repositories/tokenRepository");
const allowlist_1 = require("../methods/allowlist");
const auth_1 = require("../methods/auth");
const errors_1 = require("../types/errors");
const service_common_1 = require("@crosslab/service-common");
/**
 * This function implements the functionality for handling GET requests on /auth endpoint.
 * @param parameters The parameters of the request.
 * @throws {MissingEntityError} Thrown if token is not found in database. | Can throw errors from {@link getAllowlistedUser}. (currently all caught)
 * @throws {InconsistentDatabaseError} Thrown if multiple active keys are found.
 */
const getAuth = async (parameters) => {
    console.log(`getAuth called`);
    const HOUR = 60 * 60 * 1000;
    // Catch missing authorization header (OPTIONS requests)
    if (!parameters.Authorization) {
        return {
            status: 200,
            headers: {},
        };
    }
    // Get active key
    const activeKey = await activeKeyRepository_1.activeKeyRepository.findOneOrFail({
        where: {
            use: 'sig',
        },
    });
    const tokenString = (0, auth_1.parseBearerToken)(parameters.Authorization);
    // Non Allowlisted Auth
    try {
        // Resolve user from Authorization parameter
        const token = await tokenRepository_1.tokenRepository.findOneOrFail({
            where: {
                token: tokenString,
            },
        });
        if (!token.user) {
            throw new service_common_1.InconsistentDatabaseError(`Token has no associated user`, 500);
        }
        const user = token.user;
        // Check if token is expired
        if (token.expiresOn && new Date(token.expiresOn).getTime() < Date.now()) {
            throw new errors_1.ExpiredError(`Token is expired`, 401);
        }
        let jwt;
        const scopes = [
            ...token.scopes.map((scope) => scope.name),
            ...token.roles
                .map((role) => role.scopes.map((scope) => scope.name))
                .flat(1)
                .filter((value, index, self) => self.indexOf(value) === index),
        ].filter((value, index, self) => self.indexOf(value) === index);
        // Check if token has a device
        if (token.device) {
            // Sign device token
            console.log(`signing jwt for device ${token.device}`);
            jwt = await (0, auth_1.signDeviceToken)(token.device, user, activeKey);
        }
        else {
            // Sign user token
            console.log(`signing jwt for user ${user.username}`);
            jwt = await (0, auth_1.signUserToken)(user, activeKey, scopes);
        }
        // Update token expiration time
        if (token.expiresOn)
            token.expiresOn = new Date(Date.now() + HOUR).toISOString();
        tokenRepository_1.tokenRepository.save(token);
        console.log(`getAuth succeeded`);
        return {
            status: 200,
            headers: {
                Authorization: 'Bearer ' + jwt,
            },
        };
    }
    catch (error) {
        // Allowlisted Auth
        if (allowlist_1.allowlist[tokenString]) {
            const user = await (0, allowlist_1.getAllowlistedUser)(tokenString);
            console.log(`signing jwt for user ${user.username}`);
            const scopes = user.roles
                .map((role) => role.scopes.map((scope) => scope.name))
                .flat(1)
                .filter((value, index, self) => self.indexOf(value) === index);
            const jwt = await (0, auth_1.signUserToken)(user, activeKey, scopes);
            console.log(`getAuth succeeded`);
            return {
                status: 200,
                headers: {
                    Authorization: 'Bearer ' + jwt,
                },
            };
        }
        else {
            if (error instanceof service_common_1.MissingEntityError) {
                error.status = 401;
            }
            throw error;
        }
    }
};
exports.getAuth = getAuth;
//# sourceMappingURL=auth.js.map