"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteRolesByRoleIdUsers = exports.postRolesByRoleIdUsers = exports.getRolesByRoleIdUsers = exports.deleteRolesByRoleId = exports.patchRolesByRoleId = exports.getRolesByRoleId = exports.postRoles = exports.getRoles = void 0;
const roleRepository_1 = require("../database/repositories/roleRepository");
const userRepository_1 = require("../database/repositories/userRepository");
/**
 * This function implements the functionality for handling GET requests on /roles endpoint.
 * @param _user The user submitting the request.
 */
const getRoles = async (_user) => {
    console.log(`getRoles called`);
    const roleModels = await roleRepository_1.roleRepository.find();
    console.log(`getRoles succeeded`);
    return {
        status: 200,
        body: await Promise.all(roleModels.map(roleRepository_1.roleRepository.format)),
    };
};
exports.getRoles = getRoles;
/**
 * This function implements the functionality for handling POST requests on /roles endpoint.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 */
const postRoles = async (body, _user) => {
    console.log(`postRoles called`);
    const roleModel = await roleRepository_1.roleRepository.create(body);
    await roleRepository_1.roleRepository.save(roleModel);
    console.log(`postRoles succeeded`);
    return {
        status: 201,
        body: await roleRepository_1.roleRepository.format(roleModel),
    };
};
exports.postRoles = postRoles;
/**
 * This function implements the functionality for handling GET requests on /roles/{role_id} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if role is not found in database.
 */
const getRolesByRoleId = async (parameters, _user) => {
    console.log(`getRolesByRoleId called`);
    const roleModel = await roleRepository_1.roleRepository.findOneOrFail({
        where: {
            uuid: parameters.role_id,
        },
    });
    console.log(`getRolesByRoleId succeeded`);
    return {
        status: 200,
        body: await roleRepository_1.roleRepository.format(roleModel),
    };
};
exports.getRolesByRoleId = getRolesByRoleId;
/**
 * This function implements the functionality for handling PATCH requests on /roles/{role_id} endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if role is not found in database.
 */
const patchRolesByRoleId = async (parameters, body, _user) => {
    console.log(`patchRolesByRoleId called`);
    // TODO: check what the user is allowed to do
    const roleModel = await roleRepository_1.roleRepository.findOneOrFail({
        where: {
            uuid: parameters.role_id,
        },
    });
    await roleRepository_1.roleRepository.write(roleModel, body ?? {});
    await roleRepository_1.roleRepository.save(roleModel);
    console.log(`patchRolesByRoleId succeeded`);
    return {
        status: 200,
        body: await roleRepository_1.roleRepository.format(roleModel),
    };
};
exports.patchRolesByRoleId = patchRolesByRoleId;
/**
 * This function implements the functionality for handling DELETE requests on /roles/{role_id} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if role is not found in database.
 */
const deleteRolesByRoleId = async (parameters, _user) => {
    console.log(`deleteRolesByRoleId called`);
    // TODO: check that the user is allowed to delete the role
    const roleModel = await roleRepository_1.roleRepository.findOneOrFail({
        where: {
            uuid: parameters.role_id,
        },
    });
    await roleRepository_1.roleRepository.remove(roleModel);
    console.log(`deleteRolesByRoleId succeeded`);
    return {
        status: 204,
    };
};
exports.deleteRolesByRoleId = deleteRolesByRoleId;
/**
 * This function implements the functionality for handling GET requests on /roles/{role_id}/users endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if role is not found in database.
 */
const getRolesByRoleIdUsers = async (parameters, _user) => {
    console.log(`getRolesByRoleIdUsers called`);
    const roleModel = await roleRepository_1.roleRepository.findOneOrFail({
        where: {
            uuid: parameters.role_id,
        },
    });
    console.log(`getRolesByRoleIdUsers succeeded`);
    return {
        status: 200,
        body: await Promise.all(roleModel.users.map(userRepository_1.userRepository.format)),
    };
};
exports.getRolesByRoleIdUsers = getRolesByRoleIdUsers;
/**
 * This function implements the functionality for handling POST requests on /roles/{role_id}/users endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if role or user is not found in database.
 */
const postRolesByRoleIdUsers = async (parameters, body, _user) => {
    console.log(`postRolesByRoleIdUsers called`);
    const roleModel = await roleRepository_1.roleRepository.findOneOrFail({
        where: {
            uuid: parameters.role_id,
        },
        relations: {
            users: true,
        },
    });
    for (const userId of body ?? []) {
        const userModel = await userRepository_1.userRepository.findOneOrFail({
            where: {
                uuid: userId,
            },
        });
        roleRepository_1.roleRepository.addUserModelToRoleModel(roleModel, userModel);
    }
    await roleRepository_1.roleRepository.save(roleModel);
    console.log(`postRolesByRoleIdUsers succeeded`);
    return {
        status: 204,
    };
};
exports.postRolesByRoleIdUsers = postRolesByRoleIdUsers;
/**
 * This function implements the functionality for handling DELETE requests on /roles/{role_id}/users endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if role or user is not found in database.
 */
const deleteRolesByRoleIdUsers = async (parameters, body, _user) => {
    console.log(`deleteRolesByRoleIdUsers called`);
    const roleModel = await roleRepository_1.roleRepository.findOneOrFail({
        where: {
            uuid: parameters.role_id,
        },
        relations: {
            users: true,
        },
    });
    for (const userId of body ?? []) {
        const userModel = await userRepository_1.userRepository.findOneOrFail({
            where: {
                uuid: userId,
            },
        });
        roleRepository_1.roleRepository.removeUserModelFromRoleModel(roleModel, userModel);
    }
    await roleRepository_1.roleRepository.save(roleModel);
    console.log(`deleteRolesByRoleIdUsers succeeded`);
    return {
        status: 204,
    };
};
exports.deleteRolesByRoleIdUsers = deleteRolesByRoleIdUsers;
//# sourceMappingURL=roles.js.map