"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteUsersByUserIdRoles = exports.postUsersByUserIdRoles = exports.getUsersByUserIdRoles = exports.patchUsersByUserId = exports.deleteUsersByUserId = exports.getUsersByUserId = exports.postUsers = exports.getUsers = void 0;
const roleRepository_1 = require("../database/repositories/roleRepository");
const userRepository_1 = require("../database/repositories/userRepository");
const errors_1 = require("../types/errors");
/**
 * This function implements the functionality for handling GET requests on /users endpoint.
 * @param _user The user submitting the request.
 */
const getUsers = async (_user) => {
    console.log(`getUsers called`);
    const userModels = await userRepository_1.userRepository.find();
    console.log(`getUsers succeeded`);
    return {
        status: 200,
        body: await Promise.all(userModels.map(userRepository_1.userRepository.format)),
    };
};
exports.getUsers = getUsers;
/**
 * This function implements the functionality for handling POST requests on /users endpoint.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 */
const postUsers = async (body, _user) => {
    console.log(`postUsers called`);
    const existingUser = await userRepository_1.userRepository.findOne({
        where: {
            username: `local:${body.username}`,
        },
    });
    if (existingUser) {
        throw new errors_1.RegistrationError('User with the same username already exists', 400);
    }
    const userModel = await userRepository_1.userRepository.create({
        username: `local:${body.username}`,
        password: body.password,
    });
    await userRepository_1.userRepository.save(userModel);
    console.log(`postUsers succeeded`);
    return {
        status: 201,
        body: await userRepository_1.userRepository.format(userModel),
    };
};
exports.postUsers = postUsers;
/**
 * This function implements the functionality for handling GET requests on /users/{user_id} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if user is not found in database.
 */
const getUsersByUserId = async (parameters, _user) => {
    console.log(`getUsersByUsername called`);
    const userModel = await userRepository_1.userRepository.findOneOrFail({
        where: {
            uuid: parameters.user_id,
        },
    });
    console.log(`getUsersByUsername succeeded`);
    return {
        status: 200,
        body: await userRepository_1.userRepository.format(userModel),
    };
};
exports.getUsersByUserId = getUsersByUserId;
/**
 * This function implements the functionality for handling DELETE requests on /users/{user_id} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if user is not found in database.
 */
const deleteUsersByUserId = async (parameters, _user) => {
    console.log(`deleteUsersByUsername called`);
    const userModel = await userRepository_1.userRepository.findOneOrFail({
        where: {
            uuid: parameters.user_id,
        },
    });
    await userRepository_1.userRepository.remove(userModel);
    console.log(`deleteUsersByUsername succeeded`);
    return {
        status: 204,
    };
};
exports.deleteUsersByUserId = deleteUsersByUserId;
/**
 * This function implements the functionality for handling PATCH requests on /users/{username} endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if user is not found in database.
 */
const patchUsersByUserId = async (parameters, body, _user) => {
    console.log(`patchUsersByUsername called`);
    const userModel = await userRepository_1.userRepository.findOneOrFail({
        where: {
            uuid: parameters.user_id,
        },
    });
    await userRepository_1.userRepository.write(userModel, body ?? {});
    await userRepository_1.userRepository.save(userModel);
    console.log(`patchUsersByUsername succeeded`);
    return {
        status: 200,
        body: await userRepository_1.userRepository.format(userModel),
    };
};
exports.patchUsersByUserId = patchUsersByUserId;
/**
 * This function implements the functionality for handling GET requests on /users/{user_id}/roles endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if user is not found in database.
 */
const getUsersByUserIdRoles = async (parameters, _user) => {
    console.log(`getUsersByUserIdRoles called`);
    const userModel = await userRepository_1.userRepository.findOneOrFail({
        where: {
            uuid: parameters.user_id,
        },
    });
    await userRepository_1.userRepository.save(userModel);
    console.log(`getUsersByUserIdRoles succeeded`);
    return {
        status: 200,
        body: await Promise.all(userModel.roles.map(roleRepository_1.roleRepository.format)),
    };
};
exports.getUsersByUserIdRoles = getUsersByUserIdRoles;
/**
 * This function implements the functionality for handling POST requests on /users/{user_id}/roles endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if user or role is not found in database.
 */
const postUsersByUserIdRoles = async (parameters, body, _user) => {
    console.log(`postUsersByUserIdRoles called`);
    const userModel = await userRepository_1.userRepository.findOneOrFail({
        where: {
            uuid: parameters.user_id,
        },
    });
    for (const roleId of body ?? []) {
        const roleModel = await roleRepository_1.roleRepository.findOneOrFail({
            where: {
                uuid: roleId,
            },
        });
        userRepository_1.userRepository.addRoleModelToUserModel(userModel, roleModel);
    }
    await userRepository_1.userRepository.save(userModel);
    console.log(`postUsersByUserIdRoles succeeded`);
    return {
        status: 204,
    };
};
exports.postUsersByUserIdRoles = postUsersByUserIdRoles;
/**
 * This function implements the functionality for handling DELETE requests on /users/{user_id}/roles endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if user or role is not found in database.
 */
const deleteUsersByUserIdRoles = async (parameters, body, _user) => {
    console.log(`deleteUsersByUserIdRoles called`);
    const userModel = await userRepository_1.userRepository.findOneOrFail({
        where: {
            uuid: parameters.user_id,
        },
    });
    for (const roleId of body ?? []) {
        const roleModel = await roleRepository_1.roleRepository.findOneOrFail({
            where: {
                uuid: roleId,
            },
        });
        userRepository_1.userRepository.removeRoleModelFromUserModel(userModel, roleModel);
    }
    await userRepository_1.userRepository.save(userModel);
    console.log(`deleteUsersByUserIdRoles succeeded`);
    return {
        status: 204,
    };
};
exports.deleteUsersByUserIdRoles = deleteUsersByUserIdRoles;
//# sourceMappingURL=users.js.map