"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UninitializedRepositoryError = exports.RegistrationError = exports.OwnershipError = exports.InternalRequestError = exports.LdapBindError = exports.LdapAuthenticationError = exports.LdapError = exports.AuthenticationError = exports.ExpiredError = exports.MalformedAllowlistError = exports.DNSResolveError = void 0;
const model_1 = require("../database/model");
const types_1 = require("../generated/types");
const utils_1 = require("../methods/utils");
/**
 * This error class should be used if an error occurrs while resolving the ip of a domain.
 */
class DNSResolveError extends Error {
    constructor(message) {
        super(message);
        this.name = 'DNSResolveError';
    }
}
exports.DNSResolveError = DNSResolveError;
/**
 * This error class should be used if the allowlist is malformed.
 */
class MalformedAllowlistError extends Error {
    constructor(message) {
        super(message);
        this.name = 'MalformedAllowlistError';
    }
}
exports.MalformedAllowlistError = MalformedAllowlistError;
/**
 * This error class should be used if an object has expired.
 */
class ExpiredError extends types_1.ErrorWithStatus {
    constructor(message, status) {
        super(message, status);
        this.name = 'ExpiredError';
    }
}
exports.ExpiredError = ExpiredError;
/**
 * This error class should be used if an error occurs during authentication.
 */
class AuthenticationError extends types_1.ErrorWithStatus {
    constructor(message, status) {
        super(message, status);
        this.name = 'AuthenticationError';
    }
}
exports.AuthenticationError = AuthenticationError;
/**
 * This error class should be used if an error occurs whilst using the ldap system.
 */
class LdapError extends types_1.ErrorWithStatus {
    constructor(message, status) {
        super(message, status);
        this.name = 'LdapError';
    }
}
exports.LdapError = LdapError;
/**
 * This error class should be used if an error occurs during the authentication to the ldap system.
 */
class LdapAuthenticationError extends LdapError {
    constructor(message, status) {
        super(message, status);
        this.name = 'LdapAuthenticationError';
    }
}
exports.LdapAuthenticationError = LdapAuthenticationError;
/**
 * This error class should be used if an error occurs during the bind operation of the ldap system.
 */
class LdapBindError extends LdapError {
    constructor(message, status) {
        super(message, status);
        this.name = 'LdapBindError';
    }
}
exports.LdapBindError = LdapBindError;
/**
 * This error class should be used as a wrapper for errors thrown by the api-client.
 */
class InternalRequestError extends types_1.ErrorWithStatus {
    constructor(message, error, status) {
        super(message, status);
        this.name = 'InternalRequestError';
        this.internalError = error;
    }
}
exports.InternalRequestError = InternalRequestError;
/**
 * This error class should be used when an user is not the owner of a device.
 */
class OwnershipError extends types_1.ErrorWithStatus {
    constructor() {
        super(`User is not the owner of the device and does not have further permission`, 403);
        this.name = 'OwnershipError';
    }
}
exports.OwnershipError = OwnershipError;
/**
 * This error class should be used when an error occurs during the registration of
 * a new user.
 */
class RegistrationError extends types_1.ErrorWithStatus {
    constructor(message, status) {
        super(message, status);
        this.name = 'RegistrationError';
    }
}
exports.RegistrationError = RegistrationError;
/**
 * This error class should be used when a repository has not been initialized before use.
 */
class UninitializedRepositoryError extends types_1.ErrorWithStatus {
    constructor(model) {
        super(`${(0, utils_1.capitalizeFirstLetter)((0, model_1.getModelName)(model).toLowerCase())} repository has not been initialized`, 500);
        this.name = 'UninitializedRepositoryError';
    }
}
exports.UninitializedRepositoryError = UninitializedRepositoryError;
//# sourceMappingURL=errors.js.map